#pragma once

#include <d3dcompiler.h>
#include <d3d11.h>
#include <wrl.h>
#include <DirectXMath.h>

#include "Graphics/Device/DeviceObject.hpp"

using namespace DirectX;
using namespace Microsoft::WRL;

class GraphicsDevice;
class Framebuffer;
class Pipeline;
class SamplerState;
class BufferObject;

class CommandList : public DeviceObject
{
public:
    CommandList(GraphicsDevice* device);
    CommandList(const CommandList&) = delete;
    CommandList& operator=(const CommandList&) = delete;
    ~CommandList() = default;

    ComPtr<ID3D11DeviceContext>& GetDefferedContext() { return DeferredContext_; }

    void ExecuteCommandList();
    void BindFramebuffer(Framebuffer* framebuffer);
    void BindSamplers(SamplerState* samplerState);
    void BindPipeline(Pipeline* pipeline);
    void BindViewport(XMINT2 windowSize);
    void BindVertexBuffer(BufferObject* vertexBuffer);
    void BindIndexBuffer(BufferObject* indexBuffer);
    void BindShaders();
    void DrawIndexed(uint32_t indexCount, uint32_t startIndexLocation, int32_t baseVertexLocation);
    void ClearBuffer(XMFLOAT4 color);

    DeviceObjectType GetDeviceObjectType() override;
private:
    ComPtr<ID3D11DeviceContext> DeferredContext_;
    ComPtr<ID3D11CommandList> CommandList_;

    Framebuffer* FramebufferRef_;
    Pipeline* PipelineRef_;
};
