#include "GraphicsDevice.hpp"

#include <Graphics/GraphicsCommon.hpp>

#include <Graphics/Framebuffer/Framebuffer.hpp>
#include <Graphics/Framebuffer/FramebufferDesc.hpp>

#include <Graphics/CommandList/CommandList.hpp>

#include <Graphics/Swapchain/Swapchain.hpp>
#include <Graphics/Swapchain/SwapchainDesc.hpp>

#include <Graphics/Shader/Shader.hpp>

#include <Graphics/Sampler/SamplerState.hpp>
#include <Graphics/Pipeline/Pipeline.hpp>

#include <Resources/BufferObject/BufferObjectDesc.hpp>
#include <Resources/BufferObject/BufferObject.hpp>

#include <Graphics/Pipeline/InputAssembler/InputAssemblerDesc.hpp>
#include <Graphics/Pipeline/Blend/BlendStateDesc.hpp>
#include <Graphics/Pipeline/DepthStencil/DepthStencilStateDesc.hpp>
#include <Graphics/Pipeline/Rasterizer/RasterizerStateDesc.hpp>

GraphicsDevice::GraphicsDevice()
{
}

GraphicsDevice::~GraphicsDevice()
{
    for(auto& obj: DeviceObjects_) {
        delete obj;
    }
}

Swapchain* GraphicsDevice::CreateSwapchain(SwapchainDesc& desc)
{

    DeviceObject* obj = new Swapchain(this, desc);
    DeviceObjects_.push_back(obj);

    return static_cast<Swapchain*>(obj);
}

Shader* GraphicsDevice::CreateShader(ShaderDesc* desc)
{
    DeviceObject* obj = new Shader(desc, this);

    static_cast<Shader*>(obj)->CreateShader();
    DeviceObjects_.push_back(obj);

    return static_cast<Shader*>(obj);
}

CommandList* GraphicsDevice::CreateCommandList()
{
    DeviceObject* obj = new CommandList(this);
    DeviceObjects_.push_back(obj);

    return static_cast<CommandList*>(obj);
}

Framebuffer* GraphicsDevice::CreateFramebuffer(FrameBufferDesc& desc)
{
    DeviceObject* obj = new Framebuffer(this, desc);
    DeviceObjects_.push_back(obj);

    return static_cast<Framebuffer*>(obj);
}

void GraphicsDevice::Initialize()
{

#ifdef ENGINE_DEBUG
    DX_PRINT("Create Device&ImmediateContext",
             D3D11CreateDevice(nullptr,
                 D3D_DRIVER_TYPE_HARDWARE,
                               nullptr, D3D11_CREATE_DEVICE_DEBUG, nullptr,
                               0, D3D11_SDK_VERSION,
                               &Device_, nullptr,
                               &ImmediateContext_)
                               );
#else
    D3D11CreateDevice(nullptr, D3D_DRIVER_TYPE_HARDWARE, nullptr, 0, nullptr, 0, D3D11_SDK_VERSION, &Device_, nullptr, &ImmediateContext_);
#endif
    if (Device_ == nullptr)
        printf("Device is null!\n");
    if (ImmediateContext_ == nullptr);
         printf("Context is null!");
}

DeviceObject* GraphicsDevice::GetDeviceObject(DeviceObjectType type)
{
    auto targetObjectIt = std::find_if(DeviceObjects_.begin(), DeviceObjects_.end(),
                                       [type](DeviceObject* obj) {
                                           return obj->GetDeviceObjectType() == type;
                                       });

    if(targetObjectIt != DeviceObjects_.end())
        return *targetObjectIt;
    else
        return nullptr;
}

SamplerState* GraphicsDevice::CreateSamplerState()
{
    DeviceObject* obj = new SamplerState(this);
    DeviceObjects_.push_back(obj);

    return static_cast<SamplerState*>(obj);
}

Pipeline* GraphicsDevice::CreatePipeline(PipelineDesc& desc)
{
    DeviceObject* obj = new Pipeline(desc, *this);

    DeviceObjects_.push_back(obj);

    return static_cast<Pipeline*>(obj);
}

BufferObject* GraphicsDevice::CreateBufferObject(BufferObjectDesc& desc)
{
    DeviceObject* obj = new BufferObject(this, desc);
    DeviceObjects_.push_back(obj);

    return static_cast<BufferObject*>(obj);
}
