#pragma once

#include <d3dcompiler.h>
#include <d3d11.h>
#include <wrl.h>
#include <DirectXMath.h>

using namespace DirectX;
using namespace Microsoft::WRL;

#include <vector>

#include "Graphics/CommandList/CommandList.hpp"

class DeviceObject;
class Swapchain;
class Framebuffer;
class CommandList;
class Shader;
class SamplerState;
class Pipeline;
class BufferObject;

struct FrameBufferDesc;
struct SwapchainDesc;
struct PipelineDesc;
struct ShaderDesc;
struct BufferObjectDesc;

class GraphicsDevice
{
public:
    GraphicsDevice();
    GraphicsDevice(const GraphicsDevice& instance) = delete;
    GraphicsDevice& operator=(const GraphicsDevice& instance) = delete;
    ~GraphicsDevice();

    ComPtr<ID3D11Device>& GetDevice()
    { return Device_; }

    ComPtr<ID3D11DeviceContext>& GetImmediateContext()
    { return ImmediateContext_; }

    DeviceObject* GetDeviceObject(DeviceObjectType type);

public:
    void Initialize();

public:
    [[nodiscard]] Swapchain* CreateSwapchain(SwapchainDesc& desc);
    [[nodiscard]] CommandList* CreateCommandList();
    [[nodiscard]] Pipeline* CreatePipeline(PipelineDesc& desc);
    [[nodiscard]] Shader* CreateShader(ShaderDesc* desc);
    [[nodiscard]] SamplerState* CreateSamplerState();
    [[nodiscard]] Framebuffer* CreateFramebuffer(FrameBufferDesc& desc);
    [[nodiscard]] BufferObject* CreateBufferObject(BufferObjectDesc& desc);

private:
    std::vector<DeviceObject*> DeviceObjects_;
private:
    ComPtr<ID3D11Device> Device_;
    ComPtr<ID3D11DeviceContext> ImmediateContext_;
};

