#include "Framebuffer.hpp"

#include "Graphics/Device/GraphicsDevice.hpp"

#include <Graphics/GraphicsCommon.hpp>

Framebuffer::Framebuffer(GraphicsDevice* device,FrameBufferDesc& desc)
{
    GraphicsDevice_ = device;

    ColorAttachment_ = desc.ColorAttachment_;
    DepthAttachment_ = desc.DepthAttachment_;
}

Framebuffer::~Framebuffer()
{

}

void Framebuffer::CreateColorAttachment()
{
    D3D11_RENDER_TARGET_VIEW_DESC renderTargetViewDesc {};
    renderTargetViewDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
    renderTargetViewDesc.ViewDimension = D3D11_RTV_DIMENSION_TEXTURE2D;
    renderTargetViewDesc.Texture2D.MipSlice = 0;

#ifdef ENGINE_DEBUG
    DX_PRINT("Create RenderTargetView", GraphicsDevice_->GetDevice()->CreateRenderTargetView(ColorAttachment_.Get(), &renderTargetViewDesc, ColorAttachmentRTV_.GetAddressOf()));
#else
    GraphicsDevice_->GetDevice()->CreateRenderTargetView(ColorAttachment_.Get(), &renderTargetViewDesc, ColorAttachmentRTV_.GetAddressOf());
#endif

}

void Framebuffer::CreateDepthAttachment(XMINT2 WindowSize)
{
    D3D11_TEXTURE2D_DESC depthStencilDesc {};
    depthStencilDesc.Width = WindowSize.x;
    depthStencilDesc.Height = WindowSize.y;
    depthStencilDesc.MipLevels = 1;
    depthStencilDesc.ArraySize = 1;
    depthStencilDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
    depthStencilDesc.SampleDesc.Count = 1;
    depthStencilDesc.SampleDesc.Quality = 0;
    depthStencilDesc.Usage = D3D11_USAGE_DEFAULT;
    depthStencilDesc.BindFlags = D3D11_BIND_DEPTH_STENCIL;
    depthStencilDesc.CPUAccessFlags = 0;
    depthStencilDesc.MiscFlags = 0;

#ifdef ENGINE_DEBUG
    DX_PRINT("Create DepthStencilBuffer", GraphicsDevice_->GetDevice()->CreateTexture2D(&depthStencilDesc, nullptr, DepthAttachment_.GetAddressOf()));
#else
    GraphicsDevice_->GetDevice()->CreateTexture2D(&depthStencilDesc, nullptr, DepthAttachment_.GetAddressOf());
#endif

    D3D11_DEPTH_STENCIL_VIEW_DESC depthStencilViewDesc {};
    depthStencilViewDesc.Format = DXGI_FORMAT_D24_UNORM_S8_UINT;
    depthStencilViewDesc.ViewDimension = D3D11_DSV_DIMENSION_TEXTURE2D;
    depthStencilViewDesc.Texture2D.MipSlice = 0;

#ifdef ENGINE_DEBUG
    DX_PRINT("Create DepthStencilView", GraphicsDevice_->GetDevice()->CreateDepthStencilView(DepthAttachment_.Get(), &depthStencilViewDesc, DepthAttachmentDSV_.GetAddressOf()));
#else
    GraphicsDevice_->GetDevice()->CreateDepthStencilView(DepthAttachment_.Get(), &depthStencilViewDesc, DepthAttachmentDSV_.GetAddressOf());
#endif

}

DeviceObjectType Framebuffer::GetDeviceObjectType()
{
    return DeviceObjectType::Framebuffer;
}
