#pragma once

#include "Graphics/Device/DeviceObject.hpp"
#include <Graphics/Framebuffer/FramebufferDesc.hpp>

#include <vector>

class GraphicsDevice;

class Framebuffer : public DeviceObject
{
public:
    Framebuffer(GraphicsDevice* device, FrameBufferDesc& desc);
    Framebuffer(const Framebuffer&) = delete;
    Framebuffer& operator=(const Framebuffer&) = delete;
    virtual ~Framebuffer();

    void CreateColorAttachment();
    void CreateDepthAttachment(XMINT2 WindowSize);

    ComPtr<ID3D11RenderTargetView>& GetColorAttachmentRTV() { return ColorAttachmentRTV_; }
    ComPtr<ID3D11DepthStencilView>& GetDepthAttachmentDSV() { return DepthAttachmentDSV_; }
    ComPtr<ID3D11Texture2D>& GetColorAttachment() { return ColorAttachment_; }
    DeviceObjectType GetDeviceObjectType() override;

private:
    ComPtr<ID3D11Texture2D> ColorAttachment_;
    ComPtr<ID3D11Texture2D> DepthAttachment_;

    ComPtr<ID3D11RenderTargetView> ColorAttachmentRTV_;
    ComPtr<ID3D11DepthStencilView> DepthAttachmentDSV_;
};