#pragma once

#include <iostream>
#include <comdef.h>

#define WHITE               "\033[0;37m"
#define LOG_SUCCESS         "\033[0;32m[Success] -- "
#define LOG_WARNING         "\033[0;33m[Warning] -- "
#define LOG_ERROR           "\033[0;31m[Error]   -- "
#define LOG_INFO            "\033[0;36m[Info]    -- "
#define LOG_DISPLAY         "\033[0;37m[Display] -- "

enum class LogType : uint8_t
{
    Display,
    Info,
    Success,
    Warning,
    Error
};

#define DX_PRINT(name, x)                                   \
    do {                                                    \
        SystemLog::PrintDXRender(name, x);                  \
    } while (0)

#define LOG_PRINT(type, x)                                  \
    do {                                                    \
        SystemLog::ConsoleLog(type, x)                      \
    } while (0)

namespace SystemLog
{
static void PrintDXRender(std::string funcName, HRESULT result)
{
    LPVOID lpMsgBuf;

    DWORD bufLen = FormatMessage(
            FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
            nullptr, _com_error::HRESULTToWCode(result), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
            (LPTSTR) &lpMsgBuf, 0, nullptr);

    if(bufLen) {
        auto lpMsgStr = (LPCSTR) lpMsgBuf;
        std::string resultStr(lpMsgStr, lpMsgStr + bufLen);

        std::cout << funcName.c_str() << " -- " << resultStr.c_str();

        LocalFree(lpMsgBuf);
    }
}

template<typename... TArgs>
static void ConsoleLog(LogType type = LogType::Display, TArgs... args)
{
    std::string messageFinal{};
    switch(type) {
        case LogType::Success:
            messageFinal += LOG_SUCCESS;
            break;
        case LogType::Warning:
            messageFinal += LOG_WARNING;
            break;
        case LogType::Error:
            messageFinal += LOG_ERROR;
            break;
        case LogType::Display:
            messageFinal += LOG_DISPLAY;
            break;
        case LogType::Info:
            messageFinal += LOG_INFO;
            break;
        default:
            messageFinal += WHITE;
            break;
    }

    std::cout << messageFinal << WHITE;
    (std::cout << ... << args) << "\n";
}

static void ThrowError(const std::string& errorMessage) {
    std::cerr << "Error -- " << errorMessage << std::endl;
}

};

