#pragma once

#include <d3dcompiler.h>
#include <d3d11.h>
#include <wrl.h>
#include <DirectXMath.h>

using namespace DirectX;
using namespace Microsoft::WRL;

#include <Graphics/GraphicsCommon.hpp>
#include <Graphics/Device/DeviceObject.hpp>
#include <Graphics/Pipeline/PipelineDesc.hpp>

class GraphicsDevice;

class Pipeline : public DeviceObject
{
public:
    Pipeline(PipelineDesc& desc, GraphicsDevice& device);
    Pipeline(const Pipeline&) = delete;
    Pipeline& operator=(const Pipeline&) = delete;
    ~Pipeline() = default;

    void InitializePipeline();

    ComPtr<ID3D11InputLayout>& GetInputLayout() { return InputLayout_; }
    ComPtr<ID3D11DepthStencilState>& GetDepthStencilState() { return DepthState_; }
    ComPtr<ID3D11RasterizerState>& GetRasterizerState() { return RasterizerState_; }
    ComPtr<ID3D11BlendState>& GetBlendState() { return BlendState_; }
    ComPtr<ID3D11SamplerState>& GetSamplerState() { return SamplerState_; }
    ComPtr<ID3D11VertexShader>& GetVertexShader() { return mVertexShader; }
    ComPtr<ID3D11PixelShader>& GetPixelShader() { return mPixelShader; }

    const PipelineDesc& GetDesc() const { return Desc_; }

    DeviceObjectType GetDeviceObjectType() override;

public:
    void CreateDepthStencilState();
    void CreateRasterizerState();
    void CreateBlendState();
    
private:
    PipelineDesc Desc_;

    ComPtr<ID3D11InputLayout> InputLayout_;
    ComPtr<ID3D11DepthStencilState> DepthState_;
    ComPtr<ID3D11RasterizerState> RasterizerState_;
    ComPtr<ID3D11BlendState> BlendState_;
    ComPtr<ID3D11SamplerState> SamplerState_;
    ComPtr<ID3D11VertexShader> mVertexShader;
    ComPtr<ID3D11PixelShader> mPixelShader;
};
