#pragma once

#include "Graphics/Device/DeviceObject.hpp"

#include <d3dcompiler.h>
#include <d3d11.h>
#include <wrl.h>

using namespace Microsoft::WRL;

#include <Graphics/Shader/ShaderDesc.hpp>

class GraphicsDevice;

class Shader : public DeviceObject
{
public:
    Shader(ShaderDesc* desc, GraphicsDevice* graphicsDevice);
    Shader(const Shader& instance) = delete;
    Shader& operator=(const Shader& instance) = delete;
    ~Shader() override;

    void CreateShader();

    ComPtr<ID3D11VertexShader>& GetVertexShader() { return VertexShader_; }
    ComPtr<ID3D11PixelShader>& GetPixelShader() { return PixelShader_; }
    ComPtr<ID3D11DomainShader>& GetDomainShader() { return DomainShader_; }
    ComPtr<ID3D11HullShader>& GetHullShader() { return HullShader_; }
    ComPtr<ID3D11GeometryShader>& GetGeometryShader() { return GeometryShader_; }
    ComPtr<ID3D11ComputeShader>& GetComputeShader() { return ComputeShader_; }

    ComPtr<ID3DBlob>& GetShaderBlob() { return Blob_; }

    ShaderType GetShaderType() { return Desc_->Type_; }

    DeviceObjectType GetDeviceObjectType() override;

private:
    ComPtr<ID3D11VertexShader> VertexShader_;
    ComPtr<ID3D11DomainShader> DomainShader_;
    ComPtr<ID3D11HullShader> HullShader_;
    ComPtr<ID3D11GeometryShader> GeometryShader_;
    ComPtr<ID3D11ComputeShader> ComputeShader_;
    ComPtr<ID3D11PixelShader> PixelShader_;

    ComPtr<ID3DBlob> Blob_;

    ShaderDesc* Desc_;
};
