#include "Swapchain.hpp"
#include "Graphics/Device/GraphicsDevice.hpp"
#include "Graphics/GraphicsCommon.hpp"
#include <Definitions.h>
#include <dxgi1_5.h>

Swapchain::Swapchain(GraphicsDevice* graphicsDevice, SwapchainDesc desc) : DeviceObject()
{
    GraphicsDevice_ = graphicsDevice;
    Desc_ = desc;
}

Swapchain::~Swapchain()
{
    Swapchain_->Release();
}

void Swapchain::InitializeSwapchain()
{
    DXGI_SWAP_CHAIN_DESC SwapChainDesc = {};

    SwapChainDesc.BufferDesc.Width = Desc_.WindowSize_.x;
    SwapChainDesc.BufferDesc.Height = Desc_.WindowSize_.y;

    SwapChainDesc.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;

    SwapChainDesc.BufferDesc.RefreshRate.Numerator = 0; 
    SwapChainDesc.BufferDesc.RefreshRate.Denominator = 0;

    SwapChainDesc.BufferDesc.Scaling = DXGI_MODE_SCALING_UNSPECIFIED;
    SwapChainDesc.BufferDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;

    SwapChainDesc.SampleDesc.Count = 1;
    SwapChainDesc.SampleDesc.Quality = 0;

    SwapChainDesc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
    SwapChainDesc.BufferCount = 3;

    SwapChainDesc.OutputWindow = Desc_.WindowHandle_;
    SwapChainDesc.Windowed = true;
    SwapChainDesc.SwapEffect = DXGI_SWAP_EFFECT_FLIP_DISCARD;
    SwapChainDesc.Flags = 0;

#ifdef ENGINE_DEBUG
    printf("DXGIFactory\n");
    CreateDXGIFactory(IID_PPV_ARGS(dxgiFactory.GetAddressOf()));
    ComPtr<IDXGIFactory5> factory5;
    dxgiFactory.As(&factory5);

    bool allowTearing = false;
    factory5->CheckFeatureSupport(DXGI_FEATURE_PRESENT_ALLOW_TEARING, &allowTearing, sizeof(allowTearing));

    printf("SWAPCHAIN\n");
    DX_PRINT("Create Swapchain", dxgiFactory->CreateSwapChain(GraphicsDevice_->GetDevice().Get(), &SwapChainDesc, Swapchain_.GetAddressOf()));
#else
    GraphicsDevice_->GetDevice().As(&dxgiDevice);
    dxgiDevice->GetParent(IID_PPV_ARGS(dxgiAdapter.GetAddressOf()));
    dxgiAdapter->GetParent(IID_PPV_ARGS(dxgiFactory.GetAddressOf()));
    dxgiFactory->CreateSwapChain(GraphicsDevice_->GetDevice().Get(), &SwapChainDesc, &Swapchain_);
#endif

    if (Swapchain_ == nullptr)
        printf("Swapchain is null!");
}

ComPtr<ID3D11Texture2D> Swapchain::GetBackBuffer()
{
    ComPtr<ID3D11Texture2D>* backBuffer = new ComPtr<ID3D11Texture2D>();
    Swapchain_->GetBuffer(0, IID_PPV_ARGS(backBuffer->GetAddressOf()));
    return *backBuffer;
}

DeviceObjectType Swapchain::GetDeviceObjectType()
{
    return DeviceObjectType::Swapchain;
}
