#include "Texture.h"
#include <Graphics/Device/GraphicsDevice.hpp>
#include "stb_image.h"

Texture::Texture(const std::string& path, GraphicsDevice* pDevice)
{
	/*
	* Load texture via stbi
	*/
	int width = 0;
	int height = 0;
	int channels = 0;

	const unsigned char* pData = stbi_load(path.c_str(),&width,&height,&channels,4);
	if (pData == nullptr)
	{
		printf("Failed to load the texture");
		return;
	}

	/*
	* Create texture
	*/
	D3D11_TEXTURE2D_DESC textureDesc = {};
	textureDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
	textureDesc.Width = width;
	textureDesc.Height = height;
	textureDesc.ArraySize = 1;
	textureDesc.MipLevels = 1;
	textureDesc.MiscFlags = 0;
	textureDesc.SampleDesc.Count = 1;
	textureDesc.SampleDesc.Quality = 0;
	textureDesc.BindFlags = D3D11_BIND_SHADER_RESOURCE;
	textureDesc.CPUAccessFlags = 0;
	textureDesc.Usage = D3D11_USAGE_DEFAULT;

	D3D11_SUBRESOURCE_DATA resourceData = {};
	resourceData.pSysMem = pData;
	resourceData.SysMemPitch = width * 4;
	resourceData.SysMemSlicePitch = 0;

	if (FAILED(pDevice->GetDevice()->CreateTexture2D(&textureDesc, &resourceData, mTexture.GetAddressOf())))
	{
		printf("Failed to create the texture");
		return;
	}

	/*
	* Create srv
	*/
	D3D11_SHADER_RESOURCE_VIEW_DESC srvDesc = {};
	srvDesc.Format = textureDesc.Format;
	srvDesc.Texture2D.MipLevels = 1;
	srvDesc.Texture2D.MostDetailedMip = 0;
	srvDesc.ViewDimension = D3D11_SRV_DIMENSION_TEXTURE2D;

	if (FAILED(pDevice->GetDevice()->CreateShaderResourceView(mTexture.Get(), &srvDesc, mSrv.GetAddressOf())))
	{
		printf("Failed to create the texture");
		return;
	}
}

Texture::~Texture()
{

}
