#include "BufferObject.hpp"

#include <Graphics/GraphicsCommon.hpp>

#include <Graphics/Device/GraphicsDevice.hpp>
#include <Resources/BufferObject/BufferObjectDesc.hpp>

BufferObject::BufferObject(GraphicsDevice* device, BufferObjectDesc& desc)
{
    GraphicsDevice_ = device;
    Desc_ = desc;
}

void BufferObject::CreateBuffer()
{
    D3D11_BUFFER_DESC bufferDesc{};

    bufferDesc.ByteWidth = Desc_.ByteWidth;
    bufferDesc.Usage = static_cast<D3D11_USAGE>(Desc_.Usage);

    switch(Desc_.BindFlags) {
        case BufferObjectBindFlags::VertexBuffer:
            bufferDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
            break;
        case BufferObjectBindFlags::IndexBuffer:
            bufferDesc.BindFlags = D3D11_BIND_INDEX_BUFFER;
            break;
        case BufferObjectBindFlags::ConstantBuffer:
            bufferDesc.BindFlags = D3D11_BIND_CONSTANT_BUFFER;
            break;
        case BufferObjectBindFlags::None:
            break;
        case BufferObjectBindFlags::StructuredBuffer:
            break;
        case BufferObjectBindFlags::RawBuffer:
            break;
        case BufferObjectBindFlags::IndirectArgsBuffer:
            break;
    }

    switch(Desc_.CPUAccessFlags) {
        case BufferObjectCPUAccessFlags::Read:
            bufferDesc.CPUAccessFlags = D3D11_CPU_ACCESS_READ;
            break;
        case BufferObjectCPUAccessFlags::Write:
            bufferDesc.CPUAccessFlags = D3D11_CPU_ACCESS_WRITE;
            break;
        case BufferObjectCPUAccessFlags::None:
            bufferDesc.CPUAccessFlags = 0;
            break;
        case BufferObjectCPUAccessFlags::ReadWrite:
            break;
        case BufferObjectCPUAccessFlags::WriteDiscard:
            break;
        case BufferObjectCPUAccessFlags::WriteNoOverwrite:
            break;
    }

    switch(Desc_.MiscFlags) {
        case BufferObjectMiscFlags::None:
            bufferDesc.MiscFlags = 0;
            break;
        case BufferObjectMiscFlags::BufferStructured:
            break;
        case BufferObjectMiscFlags::BufferRaw:
            break;
        case BufferObjectMiscFlags::BufferAppend:
            break;
        case BufferObjectMiscFlags::BufferCounter:
            break;
        case BufferObjectMiscFlags::BufferStructuredAppend:
            break;
        case BufferObjectMiscFlags::BufferUAV:
            break;
        case BufferObjectMiscFlags::BufferUAVTyped:
            break;
        case BufferObjectMiscFlags::BufferUAVRaw:
            break;
        case BufferObjectMiscFlags::BufferUAVStructured:
            break;
        case BufferObjectMiscFlags::BufferUAVStructuredWithCounter:
            break;
    }

    bufferDesc.StructureByteStride = Desc_.StructureByteStride;

    D3D11_SUBRESOURCE_DATA subresourceData{};
    subresourceData.pSysMem = static_cast<const void*>(Desc_.CPUData);

#ifdef ENGINE_DEBUG
    DX_PRINT("Create Buffer", GraphicsDevice_->GetDevice()->CreateBuffer(&bufferDesc, &subresourceData, &Buffer_));
#else
    GraphicsDevice_->GetDevice()->CreateBuffer(&bufferDesc, nullptr, &Buffer_);
#endif
}
