#pragma once

template <typename T>
class Array
{
public:
    [[maybe_unused]] Array();
    [[maybe_unused]] Array(unsigned int initSize);
    ~Array() = default;

    [[maybe_unused]] T* GetSource() { return ArrayPtr; }

    [[maybe_unused]] unsigned int Length() const;
    [[maybe_unused]] unsigned int Size() const;

    [[maybe_unused]] T& At(unsigned int index);
    [[maybe_unused]] T& operator[](const unsigned index);

    [[maybe_unused]] void Add(const T& element);
    [[maybe_unused]] void InsertFirst(const T& element);
    [[maybe_unused]] void Replace(const T& element, const unsigned index);
    [[maybe_unused]] void InsertByIndex(const T& element, unsigned index);
    [[maybe_unused]] void RemoveFirst();
    [[maybe_unused]] void Remove(const unsigned index);
    [[maybe_unused]] void RemoveAll();

    [[maybe_unused]] void Swap(const unsigned& swapItemIndex1, const unsigned& swapItemIndex2);
private:

    unsigned int maximumLength = 5;
    unsigned int currentLength;
    T* ArrayPtr;

    void ChangeSize();
    void ClearMemory();
    [[maybe_unused]] unsigned MaxLength() const;
};

template<typename T>
[[maybe_unused]] void Array<T>::Replace(const T& element, const unsigned int index)
{
    if (index >= currentLength)
    {
        throw "Index is out of bounds";
    }

    ArrayPtr[index] = element;
}

template <typename T>
[[maybe_unused]] Array<T>::Array()
{
    currentLength = 0;
    ArrayPtr = new T[this->maximumLength];
}

template <typename T>
[[maybe_unused]] Array<T>::Array(unsigned int initSize)
{
    currentLength = initSize;
    maximumLength = initSize;
    ArrayPtr = new T[this->maximumLength];
}

template <typename T>
[[maybe_unused]] unsigned Array<T>::Length() const
{
    return currentLength;
}

template <typename T>
[[maybe_unused]] unsigned Array<T>::Size() const
{
    return sizeof(T) * currentLength;
}

template <typename T>
[[maybe_unused]] unsigned Array<T>::MaxLength() const
{
    return maximumLength;
}

template <typename T>
[[maybe_unused]] T& Array<T>::At(unsigned index)
{
    return ArrayPtr[index];
}

template <typename T>
[[maybe_unused]] T& Array<T>::operator[](const unsigned index)
{
    if(index >= currentLength)
        throw "Index is out of bounds";

    return ArrayPtr[index];
}

template <typename T>
[[maybe_unused]] void Array<T>::Add(const T& element)
{
    if (currentLength == maximumLength)
    {
        ChangeSize();
    }

    ArrayPtr[currentLength] = element;
    currentLength++;
}

template <typename T>
[[maybe_unused]] void Array<T>::InsertFirst(const T& element)
{
    InsertByIndex(element, 0);
}

template <typename T>
[[maybe_unused]] void Array<T>::InsertByIndex(const T& element, unsigned index)
{
    if (currentLength == maximumLength)
    {
        maximumLength *= 2;
    }

    T* Temp = new T[maximumLength];

    for (unsigned i = 0; i < currentLength; i++)
    {
        Temp[i] = ArrayPtr[i];
    }

    ClearMemory();
    ArrayPtr = new T[maximumLength];

    for (unsigned i = 0; i < index; i++)
    {
        ArrayPtr[i] = Temp[i];
    }

    ArrayPtr[index] = element;

    for (unsigned i = index; i < currentLength; i++)
    {
        ArrayPtr[i + 1] = Temp[i];
    }
    delete[] Temp;
    currentLength ++;
}

template <typename T>
[[maybe_unused]] void Array<T>::RemoveFirst()
{
    if (currentLength == 0)
    {
        throw "Array is empty";
    }

    for (unsigned i = 0; i < currentLength - 1; i++)
    {
        ArrayPtr[i] = ArrayPtr[i + 1];
    }
    currentLength--;
}

template <typename T>
[[maybe_unused]] void Array<T>::Remove(const unsigned order)
{
    if (order >= currentLength)
    {
        throw "Index is out of bounds";
    }

    for (unsigned i = order; i < currentLength; i++)
    {
        ArrayPtr[i - 1] = ArrayPtr[i];
    }
    currentLength--;
}

template <typename T>
[[maybe_unused]] void Array<T>::RemoveAll()
{
    ClearMemory();
    maximumLength = 5;
    currentLength = 0;
}

template <typename T>
[[maybe_unused]] void Array<T>::Swap(const unsigned& swapItemIndex1, const unsigned& swapItemIndex2)
{
    if (swapItemIndex1 >= currentLength || swapItemIndex1 <= currentLength)
    {
        throw "Index is out of bounds";
    }
    T TempData = ArrayPtr[swapItemIndex1];

    ArrayPtr[swapItemIndex1] = ArrayPtr[swapItemIndex2];
    ArrayPtr[swapItemIndex2] = TempData;
}

template <typename T>
[[maybe_unused]] void Array<T>::ChangeSize()
{
    T* TempData;
    if (currentLength == maximumLength)
    {
        TempData = new T[maximumLength];
        {
            for (unsigned i = 0; i < currentLength; i++)
            {
                TempData[i] = ArrayPtr[i];
            }
        }

        ClearMemory();

        maximumLength = maximumLength * 2;
        ArrayPtr = new T[maximumLength];

        for (unsigned i = 0; i < currentLength; i++)
        {
            ArrayPtr[i] = TempData[i];
        }
        delete[] TempData;
    }
}

template <typename T>
[[maybe_unused]] void Array<T>::ClearMemory()
{
    if (ArrayPtr == nullptr)
        return;

    delete[] ArrayPtr;
}

