#include <Graphics/Device/GraphicsDevice.hpp>

#include <Graphics/Framebuffer/Framebuffer.hpp>
#include <Graphics/Framebuffer/FramebufferDesc.hpp>

#include <Graphics/CommandList/CommandList.hpp>

#include <Graphics/Swapchain/Swapchain.hpp>
#include <Graphics/Swapchain/SwapchainDesc.hpp>

#include <Graphics/Shader/Shader.hpp>

#include <Graphics/Sampler/SamplerState.hpp>
#include <Graphics/Pipeline/Pipeline.hpp>
#include <Graphics/Pipeline/InputAssembler/InputAssemblerDesc.hpp>
#include <Graphics/Pipeline/Blend/BlendStateDesc.hpp>
#include <Graphics/Pipeline/DepthStencil/DepthStencilStateDesc.hpp>
#include <Graphics/Pipeline/Rasterizer/RasterizerStateDesc.hpp>

#include <Resources/BufferObject/BufferObjectDesc.hpp>
#include <Resources/BufferObject/BufferObject.hpp>
#include "C:\\Program Files (x86)\\Microsoft DirectX SDK (June 2010)\\Include\\D3DX11tex.h"
#ifdef _WIN64
#pragma comment(lib,"C:\\Program Files (x86)\\Microsoft DirectX SDK (June 2010)\\Lib\\x64\\D3DX11.lib")
#else
#pragma comment(lib,"C:\\Program Files (x86)\\Microsoft DirectX SDK (June 2010)\\Lib\\x86\\D3DX11.lib")
#endif
#include <Window/Window.h>
#include <chrono>
#include <Definitions.h>
#include <dwmapi.h>
#include <Graphics/Texture/Texture.h>

struct Vertex
{
    XMFLOAT3 Position;
    XMFLOAT4 Color;
    XMFLOAT2 Uv;
};

std::vector<uint16_t> indices = {0,1,3,1,2,3};

std::vector<Vertex> triangle =
        {
                {{-1.0f, -1.0f, 0}, {0.0f, 0.0f, 1.0f, 1.0f},{0,1} },
                {{1.0f,  -1.0f, 0}, {0.0f, 1.0f, 0.0f, 1.0f},{1,1} },
                {{1.0f,  1.0f, 0}, {0.0f, 1.0f, 0.0f, 1.0f},{1,0} },
                {{-1.0f,  1.0f, 0}, {1.0f, 0.0f, 0.0f, 1.0f},{0,0} }
        };
unsigned long long GetTimeStampUs()
{
    auto duration = std::chrono::system_clock::now().time_since_epoch();
    return std::chrono::duration_cast<std::chrono::microseconds>(duration).count();
}
int main(unsigned int argumentCount,const char** ppArguments)
{
    /*
    * Try set the priority for the process.
    */
    if (SetPriorityClass(GetCurrentProcess(), REALTIME_PRIORITY_CLASS) == 0)
    {
        printf("Failed to set Process priority to REALTIME!\n");
    }

    /*
    * Validate arguments
    * Fullscreen,windowsize,rendersize,refresh rate
    */
    printf("Checking the custom arguments...\n");
    if (argumentCount > 1)
    {
        printf("Overriding monitor index behaviour...\n");
        const char* pArgument = ppArguments[1];
        unsigned int monitorIndex = atoi(pArgument);
        Defs::MonitorIndex = monitorIndex;
        printf("Monitor index set to %d\n", Defs::MonitorIndex);
    }
    if (argumentCount > 2)
    {
        printf("Overriding dumping behaviour...\n");
        const char* pArgument = ppArguments[2];
        //Defs::bDump = strcmp(pArgument, "true") == 0;
        printf("Overriding the dumping behaviour is not supported in Cursor testing!\n", Defs::bDump);
    }
    if (argumentCount > 3)
    {
        printf("Overriding fullscreen behaviour...\n");
        const char* pArgument = ppArguments[3];
        Defs::bFullscreen = strcmp(pArgument, "true") == 0;
        //printf("Fullscreen: %d\n", Defs::bFullscreen);
        printf("Overriding the fullscreen mode is not supported in Cursor testing");

    }
    if (argumentCount > 4)
    {
        printf("Overriding window size behaviour...\n");
        const char* pArgument = ppArguments[4];
        unsigned int windowSize = atoi(pArgument);
        Defs::WindowSizeX = windowSize;
        //printf("Window Size X: %d\n", windowSize);
        printf("Overriding the window size X is not supported in Cursor testing");
    }
    if (argumentCount > 5)
    {
        printf("Overriding window size behaviour...\n");
        const char* pArgument = ppArguments[5];
        unsigned int windowSize = atoi(pArgument);
        Defs::WindowSizeY = windowSize;
        //printf("Window Size Y: %d\n", windowSize);
        printf("Overriding the window size Y is not supported in Cursor testing");
    }
    if (argumentCount > 6)
    {
        printf("Overriding rendering size behaviour...\n");
        const char* pArgument = ppArguments[6];
        unsigned int renderSize = atoi(pArgument);
        Defs::RenderSize = renderSize;
        //printf("Render Size X: %d\n", renderSize);
        printf("Overriding render size in Cursor testing is not supported!");
    }

    printf("Creating window...\n");
    Window* window = new Window("DirectX11 Renderer", {(int)Defs::WindowSizeX,(int)Defs::WindowSizeY });
    window->InitializeWindow(false);

    printf("Creating device...");
    GraphicsDevice* device = new GraphicsDevice();

    device->Initialize();

    SwapchainDesc swapchainDesc = {
            .WindowHandle_ = window->GetWindowHandle(),
            .WindowSize_ = window->GetWindowSize()
    };

    Swapchain* swapchain = device->CreateSwapchain(swapchainDesc);

    swapchain->InitializeSwapchain();

    FrameBufferDesc framebufferDesc = {
            .ColorAttachment_ = swapchain->GetBackBuffer(),
            .DepthAttachment_ = nullptr
    };

    Framebuffer* framebuffer = device->CreateFramebuffer(framebufferDesc);

    framebuffer->CreateColorAttachment();
    framebuffer->CreateDepthAttachment(window->GetWindowSize());
   
    D3D11_TEXTURE2D_DESC desc;
    ZeroMemory(&desc, sizeof(desc));
    desc.ArraySize = 1;
    desc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
    desc.Width = Defs::RenderSize;
    desc.Height = Defs::RenderSize;
    desc.MipLevels = 1;
    desc.SampleDesc.Count = 1;
    desc.SampleDesc.Quality = 0;
    desc.BindFlags = 0;
    desc.CPUAccessFlags = 0;
    desc.Usage = D3D11_USAGE_DEFAULT;

    ID3D11Texture2D* pDumpTexture = nullptr;
    device->GetDevice()->CreateTexture2D(&desc, nullptr, &pDumpTexture);

    const unsigned int pathSize = strlen(ppArguments[0]);
    const std::string exeName = "DX-Renderer.exe";
    std::string folderPath = std::string(ppArguments[0],pathSize-16);
    Texture* pBlueTexture = new Texture(folderPath + "\\blueFrameBlurred.png", device);
    Texture* pRedTexture = new Texture(folderPath + "\\redFrameBlurred.png", device);

    bool bRed = true;
    int frame = 0;
    int mousePositionIndex = 1;
    const int windowSizeX = window->GetWindowSize().x;
    const int windowSizeY = window->GetWindowSize().y;
    ID3D11DeviceContext* pImmediateContext = device->GetImmediateContext().Get();
    while(!window->ShouldClose()) 
    {
        const unsigned long long startTime = GetTimeStampUs();

        /*
        * Process window messages
        */
        window->ProcessMessage();

        /*
        * Set framebuffer
        */
        ID3D11RenderTargetView* pRtv = framebuffer->GetColorAttachmentRTV().Get();
        pImmediateContext->OMSetRenderTargets(1, &pRtv, nullptr);

        /*
        * Clear color
        */
        if (bRed)
        {
            const FLOAT clearColor[4] = { 1.0f,0.0f,0.0f,1.0f };
            pImmediateContext->ClearRenderTargetView(pRtv, clearColor);
        }
        else
        {
            const FLOAT clearColor[4] = { 0.0f,0.0f,1.0f,1.0f };
            pImmediateContext->ClearRenderTargetView(pRtv, clearColor);
        }
        bRed = !bRed;

        /*
        * Set pipeline
        */
        D3D11_VIEWPORT viewport = {};
        viewport.TopLeftX = 0;
        viewport.TopLeftY = 0;
        viewport.Width = windowSizeX;
        viewport.Height = windowSizeY;
        viewport.MinDepth = 0.0f;
        viewport.MaxDepth = 1.0f;

        /*
        * Dump
        */
        if (Defs::bDump)
        {
            pImmediateContext->CopyResource(pDumpTexture, framebuffer->GetColorAttachment().Get());

            std::string text = "image" + std::to_string(frame) + ".png";
            printf("Dump %s \n",text.c_str());

            if (FAILED(D3DX11SaveTextureToFileA(pImmediateContext, pDumpTexture, D3DX11_IFF_PNG, text.c_str())))
            {
                printf("Dump failed");
            }
        }

        /*
        * Present to swapchain
        */
        swapchain->Present();
    }

    return 0;
}

